/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Tinderbox Status Summary.
 *
 * The Initial Developer of the Original Code is
 * Myk Melez <myk@melez.com>.
 * Portions created by the Initial Developer are Copyright (C) 2003
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * Alexander J. Vincent <ajvincent@gmail.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

// This file was completely rewritten in June, 2005 by Alexander J. Vincent.

const tsOptionsURL = "chrome://tinderstatus/content/prefs.xul"

if (!Cc)
  var Cc = Components.classes;
if (!Ci)
  var Ci = Components.interfaces;

// FIXME: don't set a bunch of non-namespaced global variables.

var gXMLHttpRequest;

// The order in this list somewhat matters, as the first item is the one
// that gets loaded when you click on the statusbar icon.
var gTinderboxList  = [];
var gTinderboxListUpdateRequired = true;
var gTinderboxIndex = -1;
var gTinderboxText  = "";
var gTinderboxURIIndex = 0;
var gTinderboxStatus = null;
var gTinderboxIntervalID = 0;
var gTinderboxLastUpdate = null;

var gPrefService = Cc["@mozilla.org/preferences-service;1"]
  .getService(Ci.nsIPrefService);
var gPrefBranch = gPrefService.getBranch("extensions.tinderstatus.");

var tinderstatusPrefObserver = 
{
  register: function()
  {
    if (gPrefBranch instanceof Ci.nsIPrefBranch2) {
      gPrefBranch.addObserver("", this, false);
    }
  },

  unregister: function()
  {
    if (gPrefBranch instanceof Ci.nsIPrefBranch2) {
      gPrefBranch.removeObserver("", this);
    }
  },

  observe: function(aSubject, aTopic, aData)
  {
    if (aTopic != "nsPref:changed")
      return;

    switch (aData) {
      case "watchList":
        gTinderboxListUpdateRequired = true;
        refreshTinderboxenStatus();
        break;
      case "refreshTime":
        if (gTinderboxIntervalID)
          window.clearInterval(gTinderboxIntervalID);

        gTinderboxIntervalID = window.setInterval(refreshTinderboxenStatus,
                                                  getRefreshTime());
        break;
      case "fullList":
        setUpTinderboxMenuPopup();
        break;
    }
  }
};

function initializeTinderstatus() {
  var appInfo = Cc["@mozilla.org/xre/app-info;1"].getService(Ci.nsIXULAppInfo);

  // If we're running in SeaMonkey and the version is less than 2.*, then the
  // options dialog won't work, so disable the menu option.
  if (appInfo.ID == "{92650c4d-4b8e-4d2a-b7eb-24ecf4f6b63a}") {
    var versionChecker = Cc["@mozilla.org/xpcom/version-comparator;1"]
      .getService(Ci.nsIVersionComparator);

    if (versionChecker.compare(appInfo.version, "2.0a1pre") < 0)
      document.getElementById("tinderstatus-prefs").hidden = true;
  }

  // If we're not on windows, use preferences for the menu instead of Options.
  if (navigator.platform.toLowerCase().indexOf('win') == -1)
    document.getElementById("tinderstatus-prefs").label = "Preferences";

  // Kick off the first update.
  refreshTinderboxenStatus();

  // Setup the menu
  setUpTinderboxMenuPopup();
}

function setUpTinderboxMenuPopup() {
  // Get the popup and remove any existing items from it.
  var popup = document.getElementById("tinderstatus-context");
  var separatorItem = document.getElementById("tinderstatus-separator");
  var openTreeItem = document.getElementById("tinderstatus-open-tree");

  var nextitem = separatorItem.previousSibling;
  while (nextitem != openTreeItem) {
    popup.removeChild(nextitem);
    nextitem = separatorItem.previousSibling;
  }

  // Get the full list of tinderboxes that we know about.
  var fullList;

  try {
    fullList = gPrefBranch.getComplexValue("fullList",
                                           Ci.nsISupportsString).data;
  }
  catch (ex) {
    fullList = "Bugzilla,Camino,Firefox,Firefox3.0,Firefox-Ports,SeaMonkey,SeaMonkey-Ports,Sunbird,Thunderbird,XULRunner";
  }

  fullList = fullList.split(",");

  // Now add the tinderboxes onto the menu
  var item;
  for (var i = 0; i < fullList.length; ++i) {
    item = document.createElement("menuitem");
    item.setAttribute("label", fullList[i]);
    item.setAttribute("oncommand", "loadTinderboxPage('" + fullList[i] + "');");
    popup.insertBefore(item, separatorItem);
  }
}

function openTinderstatusPreferences() {
  var windows = Cc['@mozilla.org/appshell/window-mediator;1']
    .getService(Ci.nsIWindowMediator)
    .getEnumerator(null);

  while (windows.hasMoreElements()) {
    var win = windows.getNext();
    if (win.document.documentURI == tsOptionsURL) {
      win.focus();
      return;
    }
  }

  var features;
  try {
    var instantApply = gPrefService.getBranch(null)
      .getBoolPref("browser.preferences.instantApply");
    features = "chrome,titlebar,toolbar,centerscreen" + (instantApply ? ",dialog=no" : ",modal");
  }
  catch (e) {
    features = "chrome,titlebar,toolbar,centerscreen,modal";
  }
  openDialog(tsOptionsURL, "", features);
}

function getRefreshTime() {
  var refreshTime;

  try {
    // refreshTime is stored in minutes
    refreshTime = gPrefBranch.getIntPref("refreshTime") * 60000;

    // Always ensure refresh time is at least one minute so we don't hit
    // the servers hard
    if (refreshTime < 60000)
      refreshTime = 60000;
  }
  catch(ex) {
    // set to default of one minute.
    refreshTime = 60000;
  }

  return refreshTime;                                           
}

function updateCurrentTinderboxList() {
  try {
    gTinderboxList = gPrefBranch.getComplexValue("watchList",
                                                 Ci.nsISupportsString).data;
    gTinderboxList = gTinderboxList.split(",");

    if (gTinderboxList.length == 1 && gTinderboxList[0] == "")
      gTinderboxList = [];
  }
  catch (ex) {
    // Set to the default list
    gTinderboxList  = ["Camino",
                       "Firefox",
                       "Firefox3.0",
                       "SeaMonkey",
                       "Thunderbird",
                       "XULRunner"];
  }
  gTinderboxListUpdateRequired = false;
}

/**
 * Load into the browser a current tinderbox page.
 */
function loadTinderboxURI(evt) {
  // Only load for left or middle click events.
  if (evt.button > 1 || evt.originalTarget.localName != "statusbarpanel")
    return;

  var tinderboxToLoad;
  try {
    tinderboxToLoad = gPrefBranch.getComplexValue("gotoTree",
                                                  Ci.nsISupportsString).data;
  }
  catch (ex) {
    // If we can't get the pref, set to the default
    tinderboxToLoad = "Firefox";
  }

  if (tinderboxToLoad != "") {
    var url = "http://tinderbox.mozilla.org/" + tinderboxToLoad + "/";
    if (!handleLinkClick(evt, url, null))
      loadURI(url);
  }
}

function loadTinderboxPage(pageName) {
  loadURI("http://tinderbox.mozilla.org/" + pageName + "/");
}

function refreshTinderboxenStatus() {
  // Check we aren't currently doing an update
  if (gTinderboxIndex != -1)
    return;

  if (gTinderboxListUpdateRequired)
    updateCurrentTinderboxList();

  gTinderboxURIIndex = 0;
  getTinderboxStatus();
}

/**
 * Request a tinderbox summary from one tree.
 */
function getTinderboxStatus() {
  // If no items selected, just show nothing.
  if (gTinderboxList.length == 0) {
    document.getElementById('tinderstatus-box')
            .setAttribute("buildstatus", "none");

    setTinderboxStatus("No tinderboxes selected.");
    gTinderboxURIIndex = 0;
    return;
  }

  gTinderboxIndex++;
  if (gTinderboxIndex == gTinderboxList.length) {
    return;
  }
  var url = "http://tinderbox.mozilla.org/";
  url += gTinderboxList[gTinderboxIndex];
  url += "/quickparse.txt";

  // Oh, how I wish we didn't have to reinitialize every time.
  gXMLHttpRequest = new XMLHttpRequest();
  gXMLHttpRequest.onload = displayTinderboxStatus;
  gXMLHttpRequest.onerror = handleRequestError;
  gXMLHttpRequest.open("GET", url);
  gXMLHttpRequest.send(null);
}

function handleRequestError() {
  // Abort current attempt, wait for next refresh
  gTinderboxIndex = -1;
  gTinderboxText = "";
}

function getSetLastUpdated() {
  gTinderboxLastUpdate = new Date();

  var result = "Last Updated: ";

  if (gTinderboxLastUpdate.getHours() < 10)
    result += "0";
  
  result += gTinderboxLastUpdate.getHours();

  result += gTinderboxLastUpdate.getMinutes() < 10 ? ":0" : ":";

  result += gTinderboxLastUpdate.getMinutes();
  
  result += gTinderboxLastUpdate.getSeconds() < 10 ? ":0" : ":";

  return result + gTinderboxLastUpdate.getSeconds();
}

/**
 * Indicate via tooltip tinderboxes which show failures.
 */
function fillTinderboxStatus(aRegExp) {
  var results = gTinderboxText.match(aRegExp);

  while (gTinderboxStatus.firstChild != gTinderboxStatus.lastChild) {
    gTinderboxStatus.removeChild(gTinderboxStatus.firstChild);
  }

  gTinderboxStatus.firstChild.setAttribute("value", getSetLastUpdated());

  for (var i = 0; i < results.length; i++) {
    var node = document.createElement("description");
    node.setAttribute("value", results[i]);
    gTinderboxStatus.appendChild(node);
  }
}

/**
 * Indicate via tooltip a status message (usually success).
 */
function setTinderboxStatus(aMessage) {
  while (gTinderboxStatus.firstChild != gTinderboxStatus.lastChild) {
    gTinderboxStatus.removeChild(gTinderboxStatus.firstChild);
  }

  gTinderboxStatus.firstChild.setAttribute("value", getSetLastUpdated());

  var node = document.createElement("description");
  node.setAttribute("value", aMessage);
  gTinderboxStatus.appendChild(node);
}

/**
 * Handle response from tinderbox server, and update status bar.
 */
function displayTinderboxStatus()
{
  var text = gXMLHttpRequest.responseText + "\n";
  gTinderboxText += text;

  // Note which tinderbox fails.
  if ((gTinderboxURIIndex == 0) &&
      ((text.indexOf("busted") != -1) || (text.indexOf("testfailed") != -1))) {
    gTinderboxURIIndex = gTinderboxIndex;
  }

  // Do we need to loop around again?
  if (gTinderboxIndex < (gTinderboxList.length - 1)) {
    getTinderboxStatus();
    return;
  }

  // Change the background color of the box depending to reflect build status.
  var box = document.getElementById('tinderstatus-box');
  var re;
  if (gTinderboxText.indexOf("busted") != -1) {
    re = /.*(busted|testfailed)/g;
    fillTinderboxStatus(re);
    box.setAttribute("buildstatus", "busted");
  }
  else if (gTinderboxText.indexOf("testfailed") != -1) {
    re = /.*testfailed/g;
    fillTinderboxStatus(re);
    box.setAttribute("buildstatus", "testfailed");
  }
  else if (gTinderboxText.indexOf("success") != -1) {
    setTinderboxStatus("All tinderboxes are green.");
    box.setAttribute("buildstatus", "success");
    gTinderboxURIIndex = 0;
  }
  else {
    setTinderboxStatus("Unknown tinderbox status.");
    box.setAttribute("buildstatus", "none");
    gTinderboxURIIndex = 0;
  }

  // Change the icon superimposed on the box to reflect tree status.
  var icon = document.getElementById('tinderstatus-icon');
  if (gTinderboxText.indexOf("closed") != -1)
    icon.setAttribute("treestatus", "closed");
  else if (gTinderboxText.indexOf("open") != -1)
    icon.setAttribute("treestatus", "open");
  else
    icon.setAttribute("treestatus", "none");

  gTinderboxIndex = -1;
  gTinderboxText = "";

  // If we have just done an update, and the an update request is signalled
  // due to change of pref, run the update again
  if (gTinderboxListUpdateRequired)
    refreshTinderboxenStatus();
}

tinderstatusPrefObserver.register();

// Check the status a second after the browser loads.
window.addEventListener("load", function initTinderboxStatus(evt) {
  // Mozilla2 has been renamed to Firefox, and Firefox has been renamed
  // to Firefox3.0, so update references to those two trees in preferences.
  // Note: we only update a reference to Firefox if there's also a reference
  // to Mozilla2, since otherwise the meaning of Firefox is ambiguous
  // (it could be either the old Firefox tree or the new one).
  // FIXME: remove this code in the next release, since it only matters once,
  // and folks are likely to have upgraded by then, and the worst case is that
  // users have to update their preferences manually.
  var watchList = gPrefBranch.getCharPref("watchList").split(",");
  if (watchList.indexOf("Mozilla2") != -1) {
    if (watchList.indexOf("Firefox") != -1)
      watchList[watchList.indexOf("Firefox")] = "Firefox3.0";
    watchList[watchList.indexOf("Mozilla2")] = "Firefox";
    gPrefBranch.setCharPref("watchList", watchList.join(","));

    // We don't update the goto tree if it's Firefox, since that's the default,
    // and we assume the user wants to stay with the default and move to the new
    // tree instead of sticking with the old renamed one.
    if (gPrefBranch.getCharPref("gotoTree") == "Mozilla2")
      gPrefBranch.setCharPref("gotoTree", "Firefox");
  }

  gTinderboxStatus = document.getElementById("tinderstatus-tooltip").firstChild;
  window.setTimeout(initializeTinderstatus, 1000);

  // Wait one minute before trying again.
  gTinderboxIntervalID = window.setInterval(refreshTinderboxenStatus,
                                            getRefreshTime());
}, false);
